/****************************************************************************
** SourceFileEntry - class
**
** Created: Tue Mar 22 12:09:08 2004
**      by: Varol Okan using kate editor
**
** This class is a simple container class to store the info
** for a entry in the Source File List.
**
****************************************************************************/

#include <qfileinfo.h>
#include <qimage.h>
#include <qdom.h>
#include <qdatetime.h>

#include "global.h"
#include "utils.h"
#include "xml_dvd.h"
#include "messagebox.h"
#include "sourcefileentry.h"

AudioEntry::AudioEntry ( )
{
  iAudioNumber = -1;
}

AudioEntry &AudioEntry::operator= ( AudioEntry &theOther )
{
  iAudioNumber = theOther.iAudioNumber;
  qsIso639     = theOther.qsIso639;
  return *this;
}

Audio::Audio ( )
{
  iAudioNumber = -1;
  pTranscodeInterface = NULL;
}

Audio::~Audio ( )
{
  if ( pTranscodeInterface )
    delete pTranscodeInterface;
  pTranscodeInterface = NULL;
}

Audio &Audio::operator = ( Audio &theOther )
{
  iAudioNumber = theOther.iAudioNumber;
  qsFileName   = theOther.qsFileName;
  qsIso639     = theOther.qsIso639;
  if ( pTranscodeInterface )
    delete pTranscodeInterface;
  pTranscodeInterface = NULL;
  if ( theOther.pTranscodeInterface ) {
     pTranscodeInterface =  new TranscodeInterface;
    *pTranscodeInterface = *theOther.pTranscodeInterface;
  }
  return *this;
}

bool Audio::readProjectFile  ( QDomNode &xmlNode )
{
  QString     qsAttribute, qsFile, qsTagName;
  QDomElement theElement = xmlNode.toElement();
  QDomNode    infoNode;

  qsAttribute = theElement.attributeNode ( AUDIO_NUMBER ).value();
  if ( ! qsAttribute.isEmpty ( ) )
    iAudioNumber = qsAttribute.toInt ();
  qsAttribute = theElement.attributeNode ( AUDIO_LANGUAGE ).value();
  if ( ! qsAttribute.isEmpty ( ) )
    qsIso639 = qsAttribute;

  qsFile = theElement.text ();
  if ( ! qsFile.isEmpty ( ) )
    qsFileName = qsFile;

  infoNode = theElement.firstChild ( );
  while (!infoNode.isNull())    {
    qsTagName = infoNode.toElement ( ).tagName();
    if ( SOURCE_OBJECT_TRANSCODE == qsTagName ) {
      if ( pTranscodeInterface )
    delete pTranscodeInterface;
      pTranscodeInterface = new TranscodeInterface;
      pTranscodeInterface->readProjectFile ( infoNode );
    }
    infoNode = infoNode.nextSibling ( );
  }
  return true;
}

bool Audio::writeProjectFile ( QDomElement &inElement )
{
  QDomDocument xmlDoc = inElement.ownerDocument();
  QDomElement  tmpFileElement, theElement, transcodeElement;
  QDomText     text;

  theElement = xmlDoc.createElement ( SOURCE_OBJECT_AUDIO );

  // Here we set the attributes of the <dvdauthor> tag
  theElement.setAttribute( AUDIO_NUMBER, QString ("%1").arg ( (int)iAudioNumber ) );
  theElement.setAttribute( AUDIO_LANGUAGE,   qsIso639 );

  // store the file name too ...
  tmpFileElement = xmlDoc.createElement( AUDIO_FILE_NAME );
  text = xmlDoc.createTextNode( qsFileName );
  tmpFileElement.appendChild( text );
  theElement.appendChild ( tmpFileElement );
  if ( pTranscodeInterface )  {
    transcodeElement = xmlDoc.createElement( SOURCE_OBJECT_TRANSCODE );
    pTranscodeInterface->writeProjectFile  ( transcodeElement );
    theElement.appendChild( transcodeElement );
  }

  inElement.appendChild ( theElement );
  return true;
}

TranscodeInterface::TranscodeInterface ()
{
	iVideoBitrate = 0;
	fFrameRate    = 0.0f;
	iAudioBitrate = 0;
	iSample       = 0;
	iStartingAt   = 0;
	iEndingAt     = 1000000;  // defaulting to a large number ...
	bTranscoding  = false;
	bRemuxOnly    = false; 
	iSubtitleFromMetaInfo = -1;
}

TranscodeInterface::~TranscodeInterface ()
{

}

TranscodeInterface & TranscodeInterface::operator = (TranscodeInterface &theOther)
{
	qsVideoFormat = theOther.qsVideoFormat;
	qsVideoType   = theOther.qsVideoType;
	qsResolution  = theOther.qsResolution;
	iVideoBitrate = theOther.iVideoBitrate;
	fFrameRate    = theOther.fFrameRate;
	qsRatio       = theOther.qsRatio;
	qsAudioFormat = theOther.qsAudioFormat;
	qsAudioType   = theOther.qsAudioType;
	iAudioBitrate = theOther.iAudioBitrate;
	iSample       = theOther.iSample;
	iStartingAt   = theOther.iStartingAt;
	iEndingAt     = theOther.iEndingAt;
	bTranscoding  = theOther.bTranscoding;
	bRemuxOnly    = theOther.bRemuxOnly;
	qsTempFile    = theOther.qsTempFile;
	iSubtitleFromMetaInfo = theOther.iSubtitleFromMetaInfo;
	return *this;
}

bool TranscodeInterface::operator == (const TranscodeInterface &theOther)
{
	if ( 	( qsVideoFormat == theOther.qsVideoFormat ) && 
		( qsVideoType   == theOther.qsVideoType   ) &&
		( qsResolution  == theOther.qsResolution  ) &&
		( iVideoBitrate == theOther.iVideoBitrate ) &&
		( fFrameRate    == theOther.fFrameRate    ) &&
		( qsRatio       == theOther.qsRatio       ) &&
		( qsAudioFormat == theOther.qsAudioFormat ) &&
		( qsAudioType   == theOther.qsAudioType   ) &&
		( iAudioBitrate == theOther.iAudioBitrate ) &&
		( iSample       == theOther.iSample       ) &&
		( bRemuxOnly    == theOther.bRemuxOnly    ) &&
		( iSubtitleFromMetaInfo == theOther.iSubtitleFromMetaInfo ) &&
		( qsTempFile    == theOther.qsTempFile    ) )
		// The following can be different but the transcoding will still output the same Format.
//		( iEndingAt - iStartingAt == theOther.iEndingAt - theOther.iStartingAt ) )
		return true;
	return false;
}

bool TranscodeInterface::operator != (const TranscodeInterface &theOther)
{
	return !(*this == theOther);
}

bool TranscodeInterface::readProjectFile  (QDomNode &xmlNode)
{
  QString qsAttribute;
  QDomElement theElement = xmlNode.toElement();

  qsAttribute = theElement.attributeNode ( TRANSCODE_VIDEO_FORMAT ).value();
  if ( ! qsAttribute.isEmpty())
    qsVideoFormat = qsAttribute;
  qsAttribute = theElement.attributeNode ( TRANSCODE_VIDEO_TYPE ).value();
  if ( ! qsAttribute.isEmpty())
    qsVideoType = qsAttribute;
  qsAttribute = theElement.attributeNode ( TRANSCODE_RESOLUTION ).value();
  if ( ! qsAttribute.isEmpty())
    qsResolution = qsAttribute;
  qsAttribute = theElement.attributeNode ( TRANSCODE_VIDEO_BITRATE ).value();
  if ( ! qsAttribute.isEmpty())
    iVideoBitrate = qsAttribute.toInt ();
  qsAttribute = theElement.attributeNode ( TRANSCODE_FRAME_RATE ).value();
  if ( ! qsAttribute.isEmpty())
    fFrameRate = qsAttribute.toFloat ();
  qsAttribute = theElement.attributeNode ( TRANSCODE_RATIO ).value();
  if ( ! qsAttribute.isEmpty())
    qsRatio = qsAttribute;
  qsAttribute = theElement.attributeNode ( TRANSCODE_AUDIO_FORMAT ).value();
  if ( ! qsAttribute.isEmpty())
    qsAudioFormat = qsAttribute;
  qsAttribute = theElement.attributeNode ( TRANSCODE_AUDIO_TYPE ).value();
  if ( ! qsAttribute.isEmpty())
    qsAudioType = qsAttribute;
  qsAttribute = theElement.attributeNode ( TRANSCODE_AUDIO_BITRATE ).value();
  if ( ! qsAttribute.isEmpty())
    iAudioBitrate = qsAttribute.toInt ();
  qsAttribute = theElement.attributeNode ( TRANSCODE_SAMPLE_RATE ).value();
  if ( ! qsAttribute.isEmpty())
    iSample = qsAttribute.toInt();
  qsAttribute = theElement.attributeNode ( TRANSCODE_STARTING_AT ).value();
  if ( ! qsAttribute.isEmpty())
    iStartingAt = qsAttribute.toInt();
  qsAttribute = theElement.attributeNode ( TRANSCODE_ENDING_AT ).value();
  if ( ! qsAttribute.isEmpty())
    iEndingAt = qsAttribute.toInt ();
  qsAttribute = theElement.attributeNode ( TRANSCODE_REMUX_ONLY ).value();
  if ( ! qsAttribute.isEmpty ( ) )
    bRemuxOnly = ( qsAttribute == "true" ) ? true : false;
  qsAttribute = theElement.attributeNode ( TRANSCODE_SUBTITLE_FROM_META ).value();
  if ( ! qsAttribute.isEmpty ( ) )
    iSubtitleFromMetaInfo = qsAttribute.toInt ();
  
  // Always get the TempFileName, empty or not ... here I come ... you can't run ...
  qsTempFile = theElement.text ();    
  
  return true;
}

bool TranscodeInterface::writeProjectFile (QDomElement &theElement)
{
	// Here we set the attributes of the <TranscodeInterface> tag
	if (!qsVideoFormat.isNull())
		theElement.setAttribute( TRANSCODE_VIDEO_FORMAT, qsVideoFormat );
	if (!qsVideoType.isNull())
		theElement.setAttribute( TRANSCODE_VIDEO_TYPE, qsVideoType );
	if (!qsResolution.isNull())
		theElement.setAttribute( TRANSCODE_RESOLUTION, qsResolution );
	if (iVideoBitrate > 0)
		theElement.setAttribute( TRANSCODE_VIDEO_BITRATE, QString ("%1").arg(iVideoBitrate) );
	if (fFrameRate > 0.0)
		theElement.setAttribute( TRANSCODE_FRAME_RATE, QString ("%1").arg(fFrameRate) );
	if (!qsRatio.isNull())
		theElement.setAttribute( TRANSCODE_RATIO, qsRatio );
	if (!qsAudioFormat.isNull())
		theElement.setAttribute( TRANSCODE_AUDIO_FORMAT, qsAudioFormat );
	if (!qsAudioType.isNull())
		theElement.setAttribute( TRANSCODE_AUDIO_TYPE, qsAudioType );
	if (iAudioBitrate > 0)
		theElement.setAttribute( TRANSCODE_AUDIO_BITRATE, QString ("%1").arg(iAudioBitrate) );
	if (iSample > 0)
		theElement.setAttribute( TRANSCODE_SAMPLE_RATE, QString ("%1").arg(iSample) );
	if (iStartingAt > 0)
		theElement.setAttribute( TRANSCODE_STARTING_AT, QString ("%1").arg(iStartingAt) );
	if (iEndingAt > 0)
		theElement.setAttribute( TRANSCODE_ENDING_AT, QString ("%1").arg(iEndingAt) );
	if ( bRemuxOnly )
	  theElement.setAttribute( TRANSCODE_REMUX_ONLY, QString ("true") );
	if (iSubtitleFromMetaInfo != -1)
	  theElement.setAttribute( TRANSCODE_SUBTITLE_FROM_META, QString ("%1").arg(iSubtitleFromMetaInfo) );
	if ( ! qsTempFile.isEmpty ( ) ) {	  
	  QDomDocument xmlDoc = theElement.ownerDocument( );
	  QDomText     text   = xmlDoc.createTextNode ( qsTempFile );
	  theElement.appendChild( text );
	}

	return true;
}

SourceFileInfo::SourceFileInfo ()
{
  pPreview       = NULL;
  bAutotranscode = true;
  bMetaSubtitles = true;
  pTranscodeInterface = NULL;
  bUpdateInfo    = false;
  iPause         =  0;
  iMSecPreview   =  0;
  iTitleset      = -1;
  for ( int t=0; t<MAX_SUBTITLES; t++ )
    arraySubtitles[t] = NULL;
  for ( int t=0; t<MAX_AUDIO; t++ )
    arrayAudio[t] = NULL;
}

SourceFileInfo::~SourceFileInfo ()
{
  if (pPreview)
    delete pPreview;
  if (pTranscodeInterface)
    delete pTranscodeInterface;

  deleteSubtitles ( );
  deleteAudio     ( );
}

int SourceFileInfo::audioCount ( )
{
  int t, iCount = 0;
  for ( t=0; t<MAX_AUDIO; t++ ) {
    if ( arrayAudio[t] )
      iCount ++;
  }
  return iCount;
}

void SourceFileInfo::deleteAudio ( )
{
  int t;
  for ( t=0; t<MAX_AUDIO; t++ ) {
    if ( arrayAudio[t] )
      delete arrayAudio[t];
    arrayAudio[t] = NULL;
  }
}

void SourceFileInfo::addAudio ( Audio *pAudio )
{
  if ( ! pAudio )
    return;

  if ( arrayAudio[ pAudio->iAudioNumber ] )
    delete arrayAudio[ pAudio->iAudioNumber ];

  arrayAudio[ pAudio->iAudioNumber ] = pAudio;
}

int SourceFileInfo::subtitleCount ( )
{
  int t, iCount = 0;
  for ( t=0; t<MAX_SUBTITLES; t++ ) {
    if ( arraySubtitles[t] )
      iCount ++;
  }
  return iCount;
}

void SourceFileInfo::deleteSubtitles ( )
{
  int t;
  for ( t=0; t<MAX_SUBTITLES; t++ ) {
    if ( arraySubtitles[t] )
      delete arraySubtitles[t];
    arraySubtitles[t] = NULL;
  }
}

void SourceFileInfo::addSubtitles ( Subtitles *pSubtitle )
{
  if ( ! pSubtitle )
    return;

  if ( arraySubtitles[ pSubtitle->m_iSubtitleNumber ] )
    delete arraySubtitles[ pSubtitle->m_iSubtitleNumber ];

  arraySubtitles[ pSubtitle->m_iSubtitleNumber ] = pSubtitle;
}

// The filename for the dvdauthor.xml file depends on the pre-processing
QString SourceFileInfo::getFileName ()
{
  Utils     theUtils;
  QFileInfo fileInfo ( qsFileName );
  if ( subtitleCount ( ) > 0 ) {
    int t;
    QString qsReturn;
    for ( t=0; t<MAX_SUBTITLES; t++ ) {
      if ( arraySubtitles[t] ) // the last file name wins.
	qsReturn = arraySubtitles[t]->m_qsTempFile;
    }
    if ( qsReturn.isEmpty ( ) )
      qsReturn = QString ( "%1/subtitles.vob" ).arg ( theUtils.getTempPath ( fileInfo.baseName ( TRUE ) ) );
    return qsReturn;
  }
  if ( pTranscodeInterface ) {
    if ( ! pTranscodeInterface->qsTempFile.isEmpty () )
      return pTranscodeInterface->qsTempFile;
    else  // nothing in so we should create the file name
      return QString ( "%1/%2.mpeg2" ).arg ( theUtils.getTempPath ( fileInfo.baseName ( TRUE ) ) ).arg( fileInfo.baseName ( TRUE ) );
  }
  return qsFileName;
}

SourceFileEntry::SourceFileEntry ( )
{
  iTitleset     = -1;
  iTitle        = -1;
  iPause        =  0;
  iJumpStart    = -1;
  bIsSlideshow  = false;
  bSoundSource  = false;
  // the first audio track is used for the Video's associated audio 
  // Note: currently only the main audio track of a video is utilized.
  arrayAudioEntries[0].iAudioNumber = 0;
  arrayAudioEntries[0].qsIso639 = Global::qsLanguage;
  if ( Global::qsLanguage.length ( ) != 2 )
    arrayAudioEntries[0].qsIso639 = "en";
  // Any other audio entries are ordered by the SourceFileInfos settings.

  sizeThumbnail = QSize (150, 150);
}

SourceFileEntry::~SourceFileEntry()
{
	uint t;
	for (t=0;t<listFileInfos.count();t++)	
		delete listFileInfos[t];
}

bool SourceFileEntry::readProjectFile  (QDomNode &xmlNode)
{
        int t;
	QString qsAttribute, infoName, infoText;

	QDomNode    infoNode, transcodeNode;
	QDomElement infoElement;
	QDomElement theElement = xmlNode.toElement();

	QDomAttr a = theElement.attributeNode ( SOURCE_OBJECT_DISPLAY_NAME );
	qsDisplayName = a.value();
	a = theElement.attributeNode ( SOURCE_OBJECT_PRE );
	qsPre = a.value();
	a = theElement.attributeNode ( SOURCE_OBJECT_POST );
	qsPost = a.value();
	qsAttribute = theElement.attributeNode ( SOURCE_OBJECT_IS_SOUND_SOURCE ).value();
	bSoundSource = false;
	if ( (!qsAttribute.isEmpty ()) && (qsAttribute == QString ("true")) )
	  bSoundSource = true;
	qsAttribute = theElement.attributeNode ( SOURCE_OBJECT_PAUSE ).value();
	if (!qsAttribute.isEmpty ())
	  iPause = qsAttribute.toInt();

	for ( t=0; t<MAX_AUDIO; t++ ) {
	  arrayAudioEntries[t].iAudioNumber = -1;
	  arrayAudioEntries[t].qsIso639     = QString ( );
	}
	for ( t=0; t<MAX_SUBTITLES; t++ ) {
	  arraySubtitleEntries[t].m_iSubtitleNumber = -1;
	  arraySubtitleEntries[t].m_qsIso639        = QString ( );
	}
	  
	QDomNode sourceNode = theElement.firstChild();
	while (!sourceNode.isNull() )	{
		theElement = sourceNode.toElement();
		// Okay, here we read the stored data from the xml file.
		QString tagName = theElement.tagName();
		// Okay, this is ugly but after all it is the simplest of all xml file structure.
		// No need to get fancy ...
		if ( SOURCE_OBJECT_AUDIO == tagName )  {
		  QString qsIso639;
		  int iAudioNumber = -1;
		  qsAttribute = theElement.attributeNode ( AUDIO_LANGUAGE ).value ( );
		  if ( ! qsAttribute.isEmpty ( ) )
		    qsIso639 = qsAttribute;  
		  qsAttribute = theElement.attributeNode ( AUDIO_NUMBER ).value ( );
		  if ( ! qsAttribute.isEmpty ( ) )
		    iAudioNumber = qsAttribute.toInt ( );

		  if ( ( iAudioNumber > -1 ) && ( iAudioNumber < MAX_AUDIO ) ) {
		    arrayAudioEntries[ iAudioNumber ].iAudioNumber = iAudioNumber;
		    arrayAudioEntries[ iAudioNumber ].qsIso639     = qsIso639;
		  }		 
		}
		if ( SOURCE_OBJECT_SUBTITLES == tagName )  {
		  QString qsIso639;
		  int iSubtitleNumber = -1;
		  qsAttribute = theElement.attributeNode ( SUBTITLES_LANG ).value ( );
		  if ( ! qsAttribute.isEmpty ( ) )
		    qsIso639 = qsAttribute;  
		  qsAttribute = theElement.attributeNode ( SUBTITLES_NUMBER ).value ( );
		  if ( ! qsAttribute.isEmpty ( ) )
		    iSubtitleNumber = qsAttribute.toInt ( );

		  if ( ( iSubtitleNumber > -1 ) && ( iSubtitleNumber < MAX_SUBTITLES ) ) {
		    arraySubtitleEntries[ iSubtitleNumber ].m_iSubtitleNumber = iSubtitleNumber;
		    arraySubtitleEntries[ iSubtitleNumber ].m_qsIso639        = qsIso639;
		  }		 
		}
		else if ( SOURCE_OBJECT_INFO == tagName )  {
			SourceFileInfo *pFileInfos = new SourceFileInfo;
			qsAttribute = theElement.attributeNode (SOURCE_OBJECT_SIZE).value();
			if ( ! qsAttribute.isEmpty ( ) )
				pFileInfos->qsSize = qsAttribute;
			qsAttribute = theElement.attributeNode (SOURCE_OBJECT_RESOLUTION).value();
			if ( ! qsAttribute.isEmpty ( ) )
				pFileInfos->qsResolution = qsAttribute;
			qsAttribute = theElement.attributeNode (SOURCE_OBJECT_FPS).value();
			if ( ! qsAttribute.isEmpty ( ) )
				pFileInfos->qsFPS = qsAttribute;
			qsAttribute = theElement.attributeNode (SOURCE_OBJECT_RATIO).value();
			if ( ! qsAttribute.isEmpty ( ) )
				pFileInfos->qsRatio = qsAttribute;
			qsAttribute = theElement.attributeNode (SOURCE_OBJECT_LENGTH).value();
			if ( ! qsAttribute.isEmpty ( ) )
				pFileInfos->qsLength = qsAttribute;
			qsAttribute = theElement.attributeNode (SOURCE_OBJECT_VIDEO_FORMAT).value();
			if ( ! qsAttribute.isEmpty ( ) )
				pFileInfos->qsVideoFormat = qsAttribute;
			qsAttribute = theElement.attributeNode (SOURCE_OBJECT_AUDIO_FORMAT).value();
			if ( ! qsAttribute.isEmpty ( ) )
				pFileInfos->qsAudioFormat = qsAttribute;
			qsAttribute = theElement.attributeNode (SOURCE_OBJECT_STATUS).value();
			if ( ! qsAttribute.isEmpty ( ) )
				pFileInfos->qsStatus = qsAttribute;
			qsAttribute = theElement.attributeNode (SOURCE_OBJECT_BITS).value();
			if ( ! qsAttribute.isEmpty ( ) )
				pFileInfos->qsBits = qsAttribute;
			qsAttribute = theElement.attributeNode (SOURCE_OBJECT_SAMPLE).value();
			if ( ! qsAttribute.isEmpty ( ) )
				pFileInfos->qsSample = qsAttribute;
			qsAttribute = theElement.attributeNode (SOURCE_OBJECT_BPS).value();
			if ( ! qsAttribute.isEmpty ( ) )
				pFileInfos->qsBPS = qsAttribute;
			qsAttribute = theElement.attributeNode (SOURCE_OBJECT_PAUSE).value();
			if ( ! qsAttribute.isEmpty ( ) )
			  pFileInfos->iPause = qsAttribute.toInt();
			qsAttribute = theElement.attributeNode (SOURCE_OBJECT_TITLESET).value();
			if ( ! qsAttribute.isEmpty ( ) )
			  pFileInfos->iTitleset = qsAttribute.toInt();
			qsAttribute = theElement.attributeNode (SOURCE_OBJECT_AUTOTRANSCODE).value();
			pFileInfos->bAutotranscode = true;
			if ( ! qsAttribute.isEmpty ( ) )
			  pFileInfos->bAutotranscode = ( qsAttribute == QString ("true") );
			qsAttribute = theElement.attributeNode (SOURCE_OBJECT_METASUBTITLES).value();
			pFileInfos->bMetaSubtitles = true;
			if ( ! qsAttribute.isEmpty ( ) )
			  pFileInfos->bMetaSubtitles = ( qsAttribute == QString ("true") );
			qsAttribute = theElement.attributeNode ( SOURCE_OBJECT_MSEC_OFFSET ).value ( );
			if ( ! qsAttribute.isEmpty ( ) )
			  pFileInfos->iMSecPreview = qsAttribute.toInt ( );

			infoNode = theElement.firstChild ( );
			pFileInfos->deleteSubtitles ( );
			while (!infoNode.isNull())	{
				infoElement = infoNode.toElement();
				// Okay, here we read the stored data from the xml file.
				infoName = infoElement.tagName();
				infoText = infoElement.text ();
				if (SOURCE_OBJECT_FILE_NAME == infoName)	{
					if (!infoText.isEmpty())
						pFileInfos->qsFileName = infoText;
				}
				else if (SOURCE_OBJECT_CHAPTERS == infoName)	{
					if (!infoText.isEmpty())
						pFileInfos->listChapters = QStringList::split (",", infoText);
				}
				else if (SOURCE_OBJECT_PROPERTIES == infoName)	{
					qsAttribute = infoElement.attributeNode (SOURCE_PROPERTIES_POST).value();
					if (!qsAttribute.isEmpty())
						pFileInfos->qsPostCommand = qsAttribute;
					qsAttribute = infoElement.attributeNode (SOURCE_PROPERTIES_PRE).value();
					if (!qsAttribute.isEmpty())
						pFileInfos->qsPreCommand = qsAttribute;
					qsAttribute = infoElement.attributeNode (SOURCE_PROPERTIES_PAUSE).value();
					if (!qsAttribute.isEmpty())
						pFileInfos->iPause = qsAttribute.toInt();
				}
				else if (SOURCE_OBJECT_TRANSCODE == infoName)	{
					if ( pFileInfos->pTranscodeInterface)
						delete pFileInfos->pTranscodeInterface;
					pFileInfos->pTranscodeInterface = new TranscodeInterface;
					pFileInfos->pTranscodeInterface->readProjectFile (infoNode);
				}
				else if (SOURCE_OBJECT_SUBTITLES == infoName)	{
				  Subtitles *pSubtitle = new Subtitles;
				  pSubtitle->readProjectFile ( infoNode );
				  pFileInfos->addSubtitles   ( pSubtitle );
				}
				else if (SOURCE_OBJECT_AUDIO == infoName)	{
				  Audio *pAudio = new Audio;
				  pAudio->readProjectFile ( infoNode );
				  pFileInfos->addAudio    ( pAudio   );
				}
				infoNode = infoNode.nextSibling ();
			}
			listFileInfos.append (pFileInfos);
		}
		else if (SOURCE_OBJECT_CHAPTERS == tagName)	{
			infoText = theElement.text ();
			listChapters = QStringList::split ( QString (","), infoText);
		}
		// Otherwise go to the next node ...
		sourceNode = sourceNode.nextSibling();
	}
  // VAROL 
  /*
  printf ( "%s::%s : %d > ", __FILE__, __FUNCTION__, __LINE__ );
  for ( int e=0;e<MAX_AUDIO;e++ )
    printf ( "[%d=%s] ", arrayAudioEntries[e].iAudioNumber, arrayAudioEntries[e].qsIso639.ascii() );
  printf ( "\n" );
  */
  return true;
}

bool SourceFileEntry::writeProjectFile (QDomElement &theElement, bool bAutosave)
{
  // This check means that the slideshow has not yet been finished, thus we should not bother to
  // store this record since the mpeg does probably not exist.
  if (bIsSlideshow && bAutosave)	{
    MessageBox::warning (NULL,QObject::tr("Warning"), QObject::tr ("Warning, the slideshow \n%1\nis not finished.\nThe SourceFileEntry will not be stored.").arg(qsDisplayName), QMessageBox::Ok, QMessageBox::NoButton);
    return true;
  }
  uint t, i;
  QDomDocument xmlDoc = theElement.ownerDocument();
  QDomElement sourceNode = xmlDoc.createElement( SOURCE_OBJECT );	// <FrameObject>
  
  QDomElement fileElement, infoElement, chapterElement, transcodeElement, subtitlesElement, propertiesElement; //tag;
  QDomElement tag;
  QDomText text;
  // Here we set the attributes of the <dvdauthor> tag
  if ( ! qsDisplayName.isNull() )
    sourceNode.setAttribute( SOURCE_OBJECT_DISPLAY_NAME, qsDisplayName );
  if ( ! qsPre.isNull() )
    sourceNode.setAttribute( SOURCE_OBJECT_PRE, qsPre );
  if ( ! qsPost.isNull() )
    sourceNode.setAttribute( SOURCE_OBJECT_POST, qsPost );
  if (bSoundSource)
    sourceNode.setAttribute( SOURCE_OBJECT_IS_SOUND_SOURCE, "true" );
  sourceNode.setAttribute( SOURCE_OBJECT_PAUSE, QString ("%1").arg (iPause) ); 

  for ( t=0; t<MAX_AUDIO; t++ )  {
    if ( arrayAudioEntries [ t ].iAudioNumber > -1 ) {
      infoElement = xmlDoc.createElement( SOURCE_OBJECT_AUDIO );
      infoElement.setAttribute( AUDIO_LANGUAGE, arrayAudioEntries [ t ].qsIso639 );
      infoElement.setAttribute( AUDIO_NUMBER,   QString ( "%1" ).arg ( arrayAudioEntries [ t ].iAudioNumber ) );
      sourceNode.appendChild  ( infoElement );
    }
  }
  for ( t=0; t<MAX_SUBTITLES; t++ )  {
    if ( arraySubtitleEntries [ t ].m_iSubtitleNumber > -1 ) {
      infoElement = xmlDoc.createElement( SOURCE_OBJECT_SUBTITLES );
      infoElement.setAttribute( SUBTITLES_LANG,   arraySubtitleEntries [ t ].m_qsIso639 );
      infoElement.setAttribute( SUBTITLES_NUMBER, QString ( "%1" ).arg ( arraySubtitleEntries [ t ].m_iSubtitleNumber ) );
      sourceNode.appendChild  ( infoElement );
    }
  }
  // VAROL 
  /*
  printf ( "%s::%s : %d > ", __FILE__, __FUNCTION__, __LINE__ );
  for ( int e=0;e<MAX_AUDIO;e++ )
    printf ( "[%d=%s] ", arrayAudioEntries[e].iAudioNumber, arrayAudioEntries[e].qsIso639.ascii() );
  printf ( "\n" );
  */
  if (listFileInfos.count() > 0)	{
    for (t=0;t<listFileInfos.count();t++)	{
      infoElement = xmlDoc.createElement( SOURCE_OBJECT_INFO );
      if (!listFileInfos[t]->qsSize.isNull())
	infoElement.setAttribute( SOURCE_OBJECT_SIZE, listFileInfos[t]->qsSize );
      if (!listFileInfos[t]->qsResolution.isNull())
	infoElement.setAttribute( SOURCE_OBJECT_RESOLUTION, listFileInfos[t]->qsResolution );
      if (!listFileInfos[t]->qsFPS.isNull())
	infoElement.setAttribute( SOURCE_OBJECT_FPS, listFileInfos[t]->qsFPS );
      if (!listFileInfos[t]->qsRatio.isNull())
	infoElement.setAttribute( SOURCE_OBJECT_RATIO, listFileInfos[t]->qsRatio );
      if (!listFileInfos[t]->qsLength.isNull())
	infoElement.setAttribute( SOURCE_OBJECT_LENGTH, listFileInfos[t]->qsLength );
      if (!listFileInfos[t]->qsVideoFormat.isNull())
	infoElement.setAttribute( SOURCE_OBJECT_VIDEO_FORMAT, listFileInfos[t]->qsVideoFormat );
      if (!listFileInfos[t]->qsAudioFormat.isNull())
	infoElement.setAttribute( SOURCE_OBJECT_AUDIO_FORMAT, listFileInfos[t]->qsAudioFormat );
      if (!listFileInfos[t]->qsStatus.isNull())
	infoElement.setAttribute( SOURCE_OBJECT_STATUS, listFileInfos[t]->qsStatus );
      if (!listFileInfos[t]->qsBits.isNull())
	infoElement.setAttribute( SOURCE_OBJECT_BITS, listFileInfos[t]->qsBits );
      if (!listFileInfos[t]->qsSample.isNull())
	infoElement.setAttribute( SOURCE_OBJECT_SAMPLE, listFileInfos[t]->qsSample );
      if (!listFileInfos[t]->qsBPS.isNull())
	infoElement.setAttribute( SOURCE_OBJECT_BPS, listFileInfos[t]->qsBPS );
      if ( listFileInfos[t]->iPause !=-1 )
	infoElement.setAttribute( SOURCE_OBJECT_PAUSE, QString ("%1").arg(listFileInfos[t]->iPause) );
      if ( listFileInfos[t]->iTitleset != -1 )
	infoElement.setAttribute( SOURCE_OBJECT_TITLESET, QString ("%1").arg(listFileInfos[t]->iTitleset) );
      if ( listFileInfos[t]->bAutotranscode == false )
	infoElement.setAttribute( SOURCE_OBJECT_AUTOTRANSCODE, "false" );
      if ( listFileInfos[t]->bMetaSubtitles == false )
	infoElement.setAttribute( SOURCE_OBJECT_METASUBTITLES, "false" );
      if ( listFileInfos[t]->iMSecPreview !=-1 )
	infoElement.setAttribute( SOURCE_OBJECT_MSEC_OFFSET, QString ("%1").arg(listFileInfos[t]->iMSecPreview) );
      // Here we create another level in the XML tree for name and chapter ...
      // First the file name
      fileElement = xmlDoc.createElement( SOURCE_OBJECT_FILE_NAME );
      text = xmlDoc.createTextNode(listFileInfos[t]->qsFileName);
      fileElement.appendChild( text );
      infoElement.appendChild( fileElement );
      // Second the properties (Post / Pre, and Pause)
      propertiesElement = xmlDoc.createElement( SOURCE_OBJECT_PROPERTIES );
      if (!listFileInfos[t]->qsPostCommand.isNull())
	propertiesElement.setAttribute( SOURCE_PROPERTIES_POST, listFileInfos[t]->qsPostCommand );
      if (!listFileInfos[t]->qsPreCommand.isNull())
	propertiesElement.setAttribute( SOURCE_PROPERTIES_PRE, listFileInfos[t]->qsPreCommand );
      propertiesElement.setAttribute( SOURCE_PROPERTIES_PAUSE, QString ("%1").arg(listFileInfos[t]->iPause ));
      infoElement.appendChild( propertiesElement );
      // Then chapters if avail.
      if (listFileInfos[t]->listChapters.count() > 0)	{
	chapterElement = xmlDoc.createElement( SOURCE_OBJECT_CHAPTERS );
	text = xmlDoc.createTextNode(listFileInfos[t]->listChapters.join(QString(",")));
	chapterElement.appendChild( text );
	infoElement.appendChild( chapterElement );
      }
      // next to last the TranscodeInterface if any.
      if (listFileInfos[t]->pTranscodeInterface)	{
	transcodeElement = xmlDoc.createElement( SOURCE_OBJECT_TRANSCODE );
	listFileInfos[t]->pTranscodeInterface->writeProjectFile (transcodeElement);
	infoElement.appendChild( transcodeElement );
      }
      // Next to last we add the audio channels.
      for ( i=0; i<MAX_AUDIO; i++ ) {
	if ( listFileInfos[t]->arrayAudio[i] )
	     listFileInfos[t]->arrayAudio[i]->writeProjectFile ( infoElement );
      }
      // And finally the Subtitles
      for ( i=0; i<MAX_SUBTITLES; i++ ) {
	if ( listFileInfos[t]->arraySubtitles[i] )
	  listFileInfos[t]->arraySubtitles[i]->writeProjectFile ( infoElement );
      }
      sourceNode.appendChild( infoElement );
    }
  }
  if (listChapters.count() > 0)	{
    //printf ("SourceFileEntry::writeProjectFile <%d><%s>\n", listChapters.count (), (const char *)listChapters[0]);
    chapterElement = xmlDoc.createElement( SOURCE_OBJECT_CHAPTERS );
    sourceNode.appendChild( chapterElement );
    text = xmlDoc.createTextNode(listChapters.join(QString(",")));
    chapterElement.appendChild( text );
  }
  
  theElement.appendChild( sourceNode );
  return true;
}

SourceFileEntry *SourceFileEntry::flatClone ()
{
  // Clones this SourceFileEntries without the deep copy of the 
  // SourceFileInfos.
  SourceFileEntry *pClone = new SourceFileEntry;

  pClone->listFileInfos = listFileInfos;
  pClone->qsDisplayName = qsDisplayName;
  pClone->listChapters  = listChapters;
  pClone->bSoundSource  = bSoundSource;
  pClone->bIsSlideshow  = bIsSlideshow;
  pClone->iJumpStart    = iJumpStart;
  pClone->iTitleset     = iTitleset;
  pClone->iTitle        = iTitle;
  pClone->qsPre         = qsPre;
  pClone->qsPost        = qsPost;
  pClone->iPause        = iPause;
  for ( int t=0; t<MAX_AUDIO; t++ ) 
    pClone->arrayAudioEntries[t] = arrayAudioEntries [t];
  for ( int t=0; t<MAX_SUBTITLES; t++ ) 
    pClone->arraySubtitleEntries[t] = arraySubtitleEntries [t];
  
  return pClone;
}

int SourceFileEntry::audioCount ( bool bMaxCount )
{
  // This function returns the max audio count for this entry.
  int t, i;
  int iMaxEntryAudio = 0, iMaxInfoAudio = 0;
  SourceFileInfo *pInfo;
  for ( iMaxEntryAudio=MAX_AUDIO-1;iMaxEntryAudio>=0; iMaxEntryAudio-- ) {
    if ( arrayAudioEntries[ iMaxEntryAudio ].iAudioNumber != -1 )
      break;
  }
  if ( ! bMaxCount )
    return iMaxEntryAudio;

  // Lets say the Entry has five audio tracks but none of the FileInfos has more then three
  // In that case we should only return 3 and not 5.
  for ( t=0; t<(int)listFileInfos.count ( ); t++ ) {
    pInfo = listFileInfos [ t ];
    if ( pInfo ) {
      for ( i=MAX_AUDIO-1; i>=0; i-- ) {
	if ( pInfo->arrayAudio [ i ] ) {
	  if ( iMaxInfoAudio < i ) {
	    iMaxInfoAudio = i;
	    i=-1;
	  }
	}
      }
    }
  }
  if ( iMaxInfoAudio < iMaxEntryAudio )
    iMaxEntryAudio = iMaxInfoAudio;

  return iMaxEntryAudio;
}

int SourceFileEntry::subtitleCount ( bool bMaxCount )
{
  // This function returns the max audio count for this entry.
  int t, i;
  int iMaxEntrySubtitle = 0, iMaxInfoSubtitle = 0;
  SourceFileInfo *pInfo;
  for ( iMaxEntrySubtitle=MAX_SUBTITLES-1;iMaxEntrySubtitle>=0; iMaxEntrySubtitle-- ) {
    if ( arraySubtitleEntries[ iMaxEntrySubtitle ].m_iSubtitleNumber != -1 )
      break;
  }
  if ( ! bMaxCount )
    return iMaxEntrySubtitle;
  // Lets say the Entry has five audio tracks but none of the FileInfos has more then three
  // In that case we should only return 3 and not 5.
  for ( t=0; t<(int)listFileInfos.count ( ); t++ ) {
    pInfo = listFileInfos [ t ];
    if ( pInfo ) {
      for ( i=MAX_SUBTITLES-1; i>=0; i-- ) {
	if ( pInfo->arraySubtitles [ i ] ) {
	  if ( iMaxInfoSubtitle < i ) {
	    iMaxInfoSubtitle = i;
	    i=-1;
	  }
	}
      }
    }
  }
  if ( iMaxInfoSubtitle < iMaxEntrySubtitle )
    iMaxEntrySubtitle = iMaxInfoSubtitle;

  return iMaxEntrySubtitle;
}

SourceFileEntry & SourceFileEntry::operator = (SourceFileEntry &theOther)
{
	uint t;
	SourceFileInfo *pInfo;
	// First we take care of the entries in the list. Please note that we should keep the current ones around
	// in case we are cloning this entry.
	QValueList<SourceFileInfo *> listTemp2;
	for (t=0;t<theOther.listFileInfos.count();t++)	{
		pInfo = new SourceFileInfo ();
		*pInfo = *theOther.listFileInfos[t];
		listTemp2.append (pInfo);
	}
	// Now we should be save here to remove the entries ...	
	for (t=0;t<listFileInfos.count ();t++)
		delete listFileInfos[t];
	listFileInfos.clear();

	listFileInfos = listTemp2;

	qsDisplayName = theOther.qsDisplayName;
	listChapters  = theOther.listChapters;
	bSoundSource  = theOther.bSoundSource;
	bIsSlideshow  = theOther.bIsSlideshow;
	iJumpStart    = theOther.iJumpStart;
	iTitleset     = theOther.iTitleset;
	iTitle        = theOther.iTitle;
	qsPre         = theOther.qsPre;
	qsPost        = theOther.qsPost;
	iPause        = theOther.iPause;
	for ( t=0; t<MAX_AUDIO; t++ ) 
	  arrayAudioEntries[t] = theOther.arrayAudioEntries [t];
	for ( t=0; t<MAX_SUBTITLES; t++ ) 
	  arraySubtitleEntries[t] = theOther.arraySubtitleEntries [t];

	return *this;
}

SourceFileInfo & SourceFileInfo::operator = (SourceFileInfo &theOther)
{
  int t;
  qsFileName     = theOther.qsFileName;
  qsSize         = theOther.qsSize;
  qsResolution   = theOther.qsResolution;
  qsFPS          = theOther.qsFPS;
  qsRatio        = theOther.qsRatio;
  qsLength       = theOther.qsLength;
  qsAudioFormat  = theOther.qsAudioFormat;
  qsVideoFormat  = theOther.qsVideoFormat;
  qsStatus       = theOther.qsStatus;
  qsBits         = theOther.qsBits;
  qsSample       = theOther.qsSample;
  qsBPS          = theOther.qsBPS;
  listChapters   = theOther.listChapters;
  qsPreCommand   = theOther.qsPreCommand;
  qsPostCommand  = theOther.qsPostCommand;
  iPause         = theOther.iPause;
  iMSecPreview   = theOther.iMSecPreview;  
  iTitleset      = theOther.iTitleset;
  bAutotranscode = theOther.bAutotranscode;
  bMetaSubtitles = theOther.bMetaSubtitles;
  if (theOther.pTranscodeInterface)	{
    if (!pTranscodeInterface)
      pTranscodeInterface = new TranscodeInterface ();
    *(pTranscodeInterface) = *(theOther.pTranscodeInterface);
  }
  else if ( pTranscodeInterface )	{
    delete pTranscodeInterface;
    pTranscodeInterface = NULL;
  }
  
  deleteSubtitles ( );
  for ( t=0; t<MAX_SUBTITLES; t++ ) {
    if ( theOther.arraySubtitles [ t ] ) {
        arraySubtitles [ t ]  = new Subtitles;
      *(arraySubtitles [ t ]) = *(theOther.arraySubtitles [ t ]);
    }
  }
  deleteAudio ( );
  for ( t=0; t<MAX_AUDIO; t++ ) {
    if ( theOther.arrayAudio [ t ] ) {
        arrayAudio [ t ]  = new Audio;
      *(arrayAudio [ t ]) = *(theOther.arrayAudio [ t ]);
    }
  }
  
  return *this;
}

